<?php

namespace Drp\UserManagement\Test\Feature;


use Drp\UserManagement\Contracts\TokenGenerator;
use Drp\UserManagement\Events\AccountActivated;
use Drp\UserManagement\Events\UserCreatedWithLoginCode;
use Drp\UserManagement\Test\TestCase;
use Drp\UserManagement\User;
use Tests\Support\Fakes\FakeTokenGenerator;

class UserManagementTest extends TestCase
{

    public function setUp()
    {
        parent::setUp();
        $this->actingAsAdmin();
        app()->bind(TokenGenerator::class, FakeTokenGenerator::class);
    }

    /**
     * @test
     */
    public function as_a_standard_admin_user_I_cannot_manage_other_admin_users()
    {
        $user = factory(User::class)->create(['role' => 'Administrator']);
        $this->actingAs($user);

        $this->visitRoute('admin.index');

        $this->get(route('admin.users.index'))
            ->assertResponseStatus(403);

        $this->visitRoute('admin.index')
            ->get(route('admin.users.edit', $user->id))
            ->assertResponseStatus(403);

        $this->visitRoute('admin.index')
            ->put(route('admin.users.update', $user->id))
            ->assertResponseStatus(403);

        $this->visitRoute('admin.index')
            ->get(route('admin.users.create'))
            ->assertResponseStatus(403);

        $this->visitRoute('admin.index')
            ->post(route('admin.users.store'))
            ->assertResponseStatus(403);

        $this->visitRoute('admin.index')
            ->delete(route('admin.users.destroy', $user->id))
            ->assertResponseStatus(403);

        $this->actingAs($user)
            ->visitRoute('admin.index')
            ->dontSeeLink('Manage users', route('admin.users.index'));
    }

    /**
     * @test
     */
    public function as_a_super_admin_user_I_can_view_all_existing_users()
    {
        $user = factory(User::class)->create();
        $this->actingAsSuperAdmin()
            ->visitRoute('admin.users.index')
            ->seeLink('Add new', route('admin.users.create'))
            ->seeText($user->name)
            ->seeText($user->sso_id)
            ->seeText($user->email);
    }


    /**
     * @test
     */
    public function as_a_super_admin_user_I_can_create_a_new_admin_user()
    {
        $this->expectsEvents(UserCreatedWithLoginCode::class);

        $this->actingAsSuperAdmin()
            ->visitRoute('admin.users.create')
            ->type('New admin', 'name')
            ->type('new@admin.com', 'email')
            ->select('Administrator', 'role')
            ->press(trans('userManagement::users.create', ['label' => 'user']))
            ->seeRouteIs('admin.users.index')
            ->seeSuccessAlert(trans('userManagement::users.created', ['label' => 'user']))
            ->seeInDatabase('users', [
                'email' => 'new@admin.com',
                'name' => 'New admin',
                'role' => 'Administrator',
                'first_login_token' => 'TOKEN',
            ])
            ->seeText('New admin')
            ->seeText('new@admin.com');
    }

    /**
     * @test
     */
    public function as_a_super_admin_user_I_can_create_a_new_standard_user()
    {
        $this->expectsEvents(UserCreatedWithLoginCode::class);

        $this->actingAsSuperAdmin()
            ->visitRoute('admin.users.create')
            ->type('New user', 'name')
            ->type('new@user.com', 'email')
            ->select('User', 'role')
            ->press(trans('userManagement::users.create', ['label' => 'user']))
            ->seeRouteIs('admin.users.index')
            ->seeSuccessAlert(trans('userManagement::users.created', ['label' => 'user']))
            ->seeInDatabase('users', [
                'email' => 'new@user.com',
                'name' => 'New user',
                'role' => 'User'
            ])
            ->seeText('New user')
            ->seeText('new@user.com');

    }

    /**
     * @test
     */
    public function as_a_super_admin_user_I_can_edit_an_existing_user()
    {
        $user = factory(User::class)->create();

        $this->actingAsSuperAdmin()
            ->visitRoute('admin.users.index')
            ->click($user->name)
            ->seeRouteIs('admin.users.edit', $user->id)
            ->seeInField('name', $user->name)
            ->seeInField('email', $user->email)
            ->type('Edited user', 'name')
            ->type('edited@user.com', 'email')
            ->press(trans('userManagement::users.update', ['label' => 'user']))
            ->seeRouteIs('admin.users.index')
            ->seeSuccessAlert(trans('userManagement::users.updated', ['label' => 'user']))
            ->seeInDatabase('users', [
                'name' => 'Edited user',
                'email' => 'edited@user.com',
            ]);
    }

    /**
     * @test
     */
    public function as_a_super_admin_user_i_cant_remove_super_admin_level_from_myself()
    {
        $this->actingAsSuperAdmin();
        $this->adminUser->save();

        $this->visitRoute('admin.users.edit', $this->adminUser->id)
            ->dontSeeElement('select', ['name' => 'role']);
    }

    /**
     * @test
     */
    public function as_a_super_admin_user_I_cannot_delete_myself()
    {
        $this->actingAsSuperAdmin();
        $this->adminUser->save();

        $this->visitRoute('admin.users.edit', $this->adminUser->id)
            ->dontSee('Delete user')
            ->dontSeeLink(trans('userManagement::users.delete', ['label' => 'user']), route('admin.users.destroy', $this->adminUser->id));

        $this->delete(route('admin.users.destroy', $this->adminUser->id))
            ->assertResponseStatus(403);
    }

    /**
     * @test
     */
    public function as_a_super_admin_user_I_can_delete_another_user()
    {
        $user = factory(User::class)->create();

        $this->actingAsSuperAdmin()
            ->visitRoute('admin.users.edit', $user->id)
            ->seeText('Delete')
            ->seeLink(trans('userManagement::users.delete', ['label' => 'user']), route('admin.users.destroy', $user->id));

        $this->delete(route('admin.users.destroy', $user->id))
            ->assertRedirectedToRoute('admin.users.index')
            ->notSeeInDatabase('users', ['email' => $user->email])
            ->followRedirects()
            ->seeSuccessAlert(trans('userManagement::users.deleted', ['label' => 'user']));
    }

    /**
     * @test
     */
    public function as_a_super_admin_user_I_can_resend_a_login_code()
    {
        $this->expectsEvents(UserCreatedWithLoginCode::class);

        $user = factory(User::class)->create(['password' => null]);

        $this->actingAsSuperAdmin()
            ->visitRoute('admin.users.edit', $user->id)
            ->click('Re-send')
            ->followRedirects()
            ->seeRouteIs('admin.users.edit', $user->id)
            ->seeSuccessAlert(trans('userManagement::users.login-sent'));
    }

    /**
     * @test
     */
    public function as_a_super_admin_user_I_cant_resend_a_login_code_if_they_have_setup_their_account()
    {
        $user = factory(User::class)->create();

        $this->actingAsSuperAdmin()
            ->visitRoute('admin.users.edit', $user->id)
            ->dontSeeLink('login link', route('admin.firstTimeLogin.create', $user->id))
            ->visitRoute('admin.firstTimeLogin.create', $user->id)
            ->seeRouteIs('admin.users.edit', $user->id)
            ->seeErrorAlert(trans('userManagement::users.account-already-setup'));
    }

    /**
     * @test
     */
    public function as_a_super_admin_user_I_cannot_see_awaiting_approval_link_if_there_are_no_users_that_need_approving()
    {
        $this->actingAsSuperAdmin()
            ->visitRoute('admin.users.index')
            ->dontSeeLink(trans('userManagement::users.view-awaiting'), route('admin.users.approve.index'));
    }

    /**
     * @test
     */
    public function as_a_super_admin_user_I_can_view_all_users_awaiting_approval()
    {

        $user = factory(User::class)->create(['approved' => true]);
        $user = factory(User::class)->create(['approved' => false]);
        $user2 = factory(User::class)->create(['approved' => false]);

        $this->actingAsSuperAdmin()
            ->visitRoute('admin.users.index')
            ->seeInElement('#awaiting-count', '2')
            ->seeLink(trans('userManagement::users.view-awaiting'), route('admin.users.approve.index'))
            ->click(trans('userManagement::users.view-awaiting'))
            ->seeRouteIs('admin.users.approve.index')
            ->seeText($user->name)
            ->seeText($user->email)
            ->seeText($user2->name)
            ->seeText($user2->email);
    }

    /**
     * @test
     */
    public function as_a_super_admin_user_I_can_approve_a_user()
    {
        $this->expectsEvents(AccountActivated::class);

        $user = factory(User::class)->create(['approved' => false]);

        $this->actingAsSuperAdmin()
            ->visitRoute('admin.users.approve.index')
            ->seeText($user->name)
            ->seeText($user->email)
            ->seeInElement('.label', trans('userManagement::users.awaiting'))
            ->click($user->name)
            ->seeRouteIs('admin.users.edit', $user->id)
            ->seeText('Approve')
            ->seeLink(trans('userManagement::users.approve', ['label' => 'user']), route('admin.users.approve.update', $user->id))
            ->click(trans('userManagement::users.approve', ['label' => 'user']))
            ->seeInDatabase('users', [
                'id' => $user->id,
                'approved' => 1
            ])
            ->seeRouteIs('admin.users.edit', $user->id)
            ->seeSuccessAlert(trans('userManagement::users.approved', ['label' => 'user']));
    }
}
